<?php

namespace Drupal\node_preview_save\Form;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\Form\NodePreviewForm;

/**
 * Node preview form that allows saving and downloading node as PDF.
 */
class NodePreviewSaveForm extends NodePreviewForm {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'node_preview_form_select_save';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, EntityInterface $node = NULL) {
    $form = parent::buildForm($form, $form_state, $node);

    $form['save_preview'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#attributes' => ['class' => ['node-preview-save']],
      '#weight' => 0,
      '#options' => [
        'node_id' => $node->id(),
        'node_type' => $node->bundle(),
        'node_revision' => $node->getRevisionId(),
      ],
      '#submit' => [[$this, 'saveNode']],
    ];

    return $form;
  }

  /**
   * Saves the node being previewed and redirects to node page.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function saveNode(array $form, FormStateInterface $form_state): void {
    $store = \Drupal::service('tempstore.private')->get('node_preview');

    $preview = $store->get($form_state->getValue('uuid'));

    $node_preview = $preview->getFormObject()->getEntity();
    $node_preview->save();

    $form_state->setIgnoreDestination();
    $form_state->setRedirectUrl($node_preview->toUrl());
    $this->messenger()->addMessage($this->t('Node has been saved.'));
  }

}
